###########################################################
## This is an R file that forms part of the project
## "Estimation of log-GARCH models in the presence of zero
## returns" by Genaro Sucarrat and Alvaro Escribano.
##
## The file contains the code that undertakes the analyses
## contained in Section 4.2, and was first created 7 July
## 2014, Getafe (Spain).
##
## CONTENTS:
##
## 1 INITIATE
## 2 APPLE ANALYSIS
##
###########################################################


###########################################################
## 1 INITIATE
###########################################################

##remove everything in environment:
rm(list=ls())

##set working directory:
#setwd("C:/Users/sucarrat/Documents/projects/zerologgarch/analysis/20160912")
setwd(choose.dir())

##load lgarch version 0.5:
#source("C:/Users/sucarrat/Documents/R/gs/lgarch/20140901build/20140901lgarch.Rpackage.R")
source("20140901lgarch.Rpackage.R")

##load acl-code (version 20141215):
source("20141215acl.Rpackage.R")

##load the gets library (needed for the info.criterion function):
library(gets)


###########################################################
## 2 APPLE ANALYSIS
###########################################################

mapple <- read.csv("table_apple.csv")
apple <- zoo(mapple[,"Adj.Close"], order.by=as.Date(mapple[,1], "%Y-%m-%d"))
Dlapple <- diff(log(apple))*100
vIapple <- zoo(as.numeric(Dlapple != 0), order.by=index(Dlapple))
vTapple <- zoo(1:length(vIapple)/length(vIapple),
  order.by=index(vIapple))
vLogTapple <- zoo(log(1:length(vIapple)), order.by=index(vIapple))
volapple <- zoo(mapple[,"Volume"], order.by=as.Date(mapple[,1], "%Y-%m-%d"))
lvolapple <- log(volapple)
Dlvolapple <- diff(log(volapple))

#zero-adjusted return (zeros replaced by small non-zero value):
Dlapple.where.zeros <- which( Dlapple == 0)
Dlapple.zero.eps <- min(abs(Dlapple[-Dlapple.where.zeros]))
Dlappleadj <- Dlapple
Dlappleadj[Dlapple.where.zeros] <- Dlapple.zero.eps

##plot of price, return, I, volume and log-volume:
which2Jan1985 <- which(index(apple[-1])=="1985-01-02")
which2Jan1990 <- which(index(apple[-1])=="1990-01-02")
which3Jan1995 <- which(index(apple[-1])=="1995-01-03")
which3Jan2000 <- which(index(apple[-1])=="2000-01-03")
which3Jan2005 <- which(index(apple[-1])=="2005-01-03")
which4Jan2010 <- which(index(apple[-1])=="2010-01-04")
whereYrs <- c(which2Jan1985, which2Jan1990, which3Jan1995,
  which3Jan2000, which3Jan2005, which4Jan2010)
labelYrs <- c("1985", "1990", "1995", "2000", "2005", "2010")

  ##record current graph-values:
  def.par <- par(no.readonly=TRUE)
  
    par( mai=c(0,0.8,0,0) ) #c(bottom, left, top, right)
#  par( mai=c(0,0.6,0,0) ) #c(bottom, left, top, right)

    par(fig=c(0,0.99,0.82,1)) #c(x-start,x-end,y-start,y-end)
    plot(1:length(apple[-1]), coredata(apple[-1]), main="",
      xlab="", ylab="Price (USD)", cex.lab=0.8, type="l", col="blue",
      lwd=1, yaxt="n", xaxt="n")
    axis(side=2, cex.axis=0.7, lwd.ticks=2) #1=below, 2=left, 3=above and 4=right

    par(fig=c(0,0.99,0.64,0.82), new=TRUE)
    plot(1:length(apple[-1]), coredata(Dlapple), main="",
      xlab="", ylab="Log-return (in %)", cex.lab=0.8, type="l",
      col="blue", lwd=1, yaxt="n", xaxt="n")
    axis(side=2, cex.axis=0.7, lwd.ticks=2) #1=below, 2=left, 3=above and 4=right

    par(fig=c(0,0.99,0.46,0.64), new=TRUE)
    plot(1:length(apple[-1]), coredata(vIapple), main="",
      xlab="", ylab="I", cex.lab=0.8, type="l",
      col="blue", yaxt="n", xaxt="n")
    axis(side=2, cex.axis=0.7, lwd.ticks=2) #1=below, 2=left, 3=above and 4=right

    par(fig=c(0,0.99,0.28,0.46), new=TRUE)
    plot(1:length(apple[-1]), coredata(volapple[-1]), main="",
      xlab="", ylab="Volume", cex.lab=0.8, type="l",
      col="blue", yaxt="n", xaxt="n")
    axis(side=2, cex.axis=0.7, lwd.ticks=2) #1=below, 2=left, 3=above and 4=right

    par(fig=c(0,0.99,0.10,0.28), new=TRUE)
    plot(1:length(apple[-1]), coredata(lvolapple[-1]), main="",
      xlab="", ylab="Log(volume)", cex.lab=0.8, type="l",
      col="blue", yaxt="n", xaxt="n")
    axis(side=2, cex.axis=0.7, lwd.ticks=2) #1=below, 2=left, 3=above and 4=right
    axis(side=1, at=whereYrs, cex.axis=0.7, lwd.ticks=2, labels=FALSE)
    axis(side=1, at=whereYrs, cex.axis=0.7, lwd.ticks=2,
      labels=labelYrs)

  ##return to default margin sizes:
  par(def.par, no.readonly=FALSE)

##plot some series (not in paper!):
appleGraph01 <- cbind(apple[-1], Dlapple, vIapple[-1],
  volapple[-1], lvolapple[-1])
plot(appleGraph01, col="blue", main="The Apple stock from 10 September 1984 to 23 August 2013",
  xlab="", ylab=c("Price (USD)", "Log-return (in %)", "I",
  "Volume", "Log(volume)"))

##zero-probability model 1: constant
vIapple.Mod01 <- acl(vIapple, ma=0, ar=0)
vIapple.Mod01Fit <- fitted(vIapple.Mod01, verbose=TRUE)
vIapple.Mod01Logl <- as.numeric(logLik(vIapple.Mod01))
vIapple.Mod01AIC <- info.criterion(vIapple.Mod01Logl,
  n=length(vIapple), k=1, method="aic")
vIapple.Mod01BIC <- info.criterion(vIapple.Mod01Logl,
  n=length(vIapple), k=1, method="sc")

##zero-probability model 2: deterministic trend
vIapple.Mod02 <- acl(vIapple, ma=0, ar=0, xreg=vTapple)
vIapple.Mod02Fit <- fitted(vIapple.Mod02, verbose=TRUE)
vIapple.Mod02Logl <- as.numeric(logLik(vIapple.Mod02))
vIapple.Mod02AIC <- info.criterion(vIapple.Mod02Logl,
  n=length(vIapple), k=2, method="aic")
vIapple.Mod02BIC <- info.criterion(vIapple.Mod02Logl,
  n=length(vIapple), k=2, method="sc")
  
##zero-probability model 3: deterministic log-trend
## (not in paper!)
vIapple.Mod03 <- acl(vIapple, ma=0, ar=0, xreg=vLogTapple)
vIapple.Mod03Fit <- fitted(vIapple.Mod03, verbose=TRUE)

##zero-probability model 4: log-volume
vIapple.Mod04 <- acl(vIapple, ma=0, ar=0, xreg=lvolapple[-1])
vIapple.Mod04Fit <- fitted(vIapple.Mod04, verbose=TRUE)
vIapple.Mod04Logl <- as.numeric(logLik(vIapple.Mod04))
#OLD: vIapple.Mod04Logl <- as.numeric(logLik(vIapple.Mod02))
vIapple.Mod04AIC <- info.criterion(vIapple.Mod04Logl,
  n=length(vIapple), k=2, method="aic")
vIapple.Mod04BIC <- info.criterion(vIapple.Mod04Logl,
  n=length(vIapple), k=2, method="sc")
  
##zero-probability model 5: deterministic trend and log-volume
## (not in paper!)
vIapple.Mod05 <- acl(vIapple, ma=0, ar=0,
  xreg=cbind(vTapple, lvolapple[-1]))
vIapple.Mod05Fit <- fitted(vIapple.Mod04, verbose=TRUE)

##zero-probability model 6: log of price-level
vIapple.Mod06 <- acl(vIapple, ma=0, ar=0, xreg=log(apple[-1]))
vIapple.Mod06Fit <- fitted(vIapple.Mod06, verbose=TRUE)
vIapple.Mod06Logl <- as.numeric(logLik(vIapple.Mod06))
vIapple.Mod06AIC <- info.criterion(vIapple.Mod06Logl,
  n=length(vIapple), k=2, method="aic")
vIapple.Mod06BIC <- info.criterion(vIapple.Mod06Logl,
  n=length(vIapple), k=2, method="sc")

#make X-variables:
DlappleAsym <- log(Dlappleadj^2)*(Dlapple < 0)
DlappleAsym <- glag(DlappleAsym, pad.value=0) ##asymmetry term:
lvolapplelagged <- glag(lvolapple[-1],
  pad.value=15.5)
vIapplelagged <- glag(vIapple, pad.value=1)
hthatlagged <- glag(vIapple.Mod04Fit[,"h"],
  pad.value=mean(vIapple.Mod04Fit[,"h"]))
Tstar <- sum(vIapple)

#log-garch(0,0):
Dlapple.Mod00 <- lgarch(Dlapple, arch=0, garch=0, method="ml")
Dlapple.Mod00Fit <- fitted(Dlapple.Mod00, verbose=TRUE)
Dlapple.Mod00Logl <- as.numeric(logLik(Dlapple.Mod00))
Dlapple.Mod00AIC <- info.criterion(Dlapple.Mod00Logl,
  n=Tstar, k=1, method="aic")
Dlapple.Mod00BIC <- info.criterion(Dlapple.Mod00Logl,
  n=Tstar, k=1, method="sc")

#asymmetric log-garch(1,1):
Dlapple.Mod01 <- lgarch(Dlapple, xreg=DlappleAsym, method="ml")
Dlapple.Mod01Fit <- fitted(Dlapple.Mod01, verbose=TRUE)
Dlapple.Mod01Logl <- as.numeric(logLik(Dlapple.Mod01))
Dlapple.Mod01AIC <- info.criterion(Dlapple.Mod01Logl,
  n=Tstar, k=4, method="aic")
Dlapple.Mod01BIC <- info.criterion(Dlapple.Mod01Logl,
  n=Tstar, k=4, method="sc")

#asymmetric log-garch(1,1)-V:
Dlapple.Mod02 <- lgarch(Dlapple,
  xreg=cbind(DlappleAsym, lvolapplelagged), method="ml")
Dlapple.Mod02Fit <- fitted(Dlapple.Mod02, verbose=TRUE)
Dlapple.Mod02Logl <- as.numeric(logLik(Dlapple.Mod02))
Dlapple.Mod02AIC <- info.criterion(Dlapple.Mod02Logl,
  n=Tstar, k=5, method="aic")
Dlapple.Mod02BIC <- info.criterion(Dlapple.Mod02Logl,
  n=Tstar, k=5, method="sc")

#asymmetric log-garch(1,1)-V,I,h:
Dlapple.Mod03 <- lgarch(Dlapple,
  xreg=cbind(DlappleAsym,lvolapplelagged,vIapplelagged,hthatlagged),
  method="ml")
Dlapple.Mod03Fit <- fitted(Dlapple.Mod03, verbose=TRUE)
Dlapple.Mod03Logl <- as.numeric(logLik(Dlapple.Mod03))
Dlapple.Mod03AIC <- info.criterion(Dlapple.Mod03Logl,
  n=Tstar, k=7, method="aic")
Dlapple.Mod03BIC <- info.criterion(Dlapple.Mod03Logl,
  n=Tstar, k=7, method="sc")

#Dlapple.Mod04 <- lgarch(Dlapple[-1],
#  xreg=cbind(glag(vIapple.Mod05Fit[,"h"], pad=FALSE),
#  glag(Dlvolapple, pad=FALSE)), method="ml")
#Dlapple.Mod04Fit <- fitted(Dlapple.Mod04, verbose=TRUE)
#Dlapple.Mod04Logl <- logLik(Dlapple.Mod03)
#Dlapple.Mod04AIC <- info.criterion(Dlapple.Mod04Logl,
#  n=length(Dlapple[-1]), k=5, method="aic")
#Dlapple.Mod04BIC <- info.criterion(Dlapple.Mod04Logl,
#  n=length(Dlapple[-1]), k=5, method="sc")
